<?php

/**
 * \file Traitement.class.inc
 * \brief Traitement.class.inc \n \n Ce fichier contient la classe php Traitement.
 *
 * Cette classe virtuelle permet de générer les éléments nécessaires à la création d'un rapport ou
 * à l'éxécution d'un traitement FME.
 *
 * Cette classe permet de générer des fichiers textes, csv ou d'images pour la création de rapport.
 * Elle permet aussi de lancer la création d'un rapport ou l'éxécution d'un traitement FME.
 *
 * \author Fabien Marty <fabien.marty@veremes.com>
 * \update Matthieu Ambrosy <matthieu.ambrosy@veremes.com>
 */
require_once ("vmlib/phpUtil.inc");
require_once ("vmlib/logUtil.inc");
require_once ("vmlib/stringUtil.inc");
require_once ("vmlib/cryptUtil.inc");
require_once ("gtf_lib/GtfFmwParser.class.inc");
require_once ("vmlib/Email.class.inc");
require_once ("gtf_lib/gtf_object/Order.class.inc");
require_once 'string.inc';

class Traitement {

    //Variable stockant l'url du log FME
    var $sLogFme = "";

    /**
     * Identifiant de la demande.
     */
    var $iDemandeId;

    /**
     * Fichier de log.
     */
    var $sRobotLogFile;

    /**
     * Objet de la classe Bd. Connexion à la base de données
     */
    var $oBd;

    /**
     * Boolean spécifiant si le traitement est en erreur ou pas.
     */
    var $bErreur = false;

    /**
     * Boolean spécifiant si FME a subit un crash.
     */
    var $bFmeCrash = false;

    /**
     * Message d'erreur.
     */
    var $sMessageErreur;

    /**
     * Tableau des paramètres
     */
    var $aParametre;

    /**
     * Chaine contenant les parametres
     */
    var $sParams;

    /**
     * Emplacement du fichier projet FMW
     */
    var $sFmwFileName;

    /**
     * Tableau des propriétés provenant du fichier properties_robot.inc
     */
    var $aProperties;

    /**
     * Tableau des extensions associées à une source de données GUI
     */
    var $aExtension;
    var $sDestination;
    var $sSource;
    var $sSourceTemp = "";

    /**
     * Paramètres de cryptage du résultat.
     */
    var $bDataEncrypt = false;
    var $sSecretKey;
    
    /**
    Pramaètre pour modifir la GTF_CONNECTION_STRING
    */
    var $bUseExternalDbConnection = false;
    
    /**
     * Formatage des paramètres différent si le traitement est envoyé vers Fme Server.
     */
    var $bFmeServer = false;
    
    /**
     * Nom de répertoire du résultat de la demande.
     */
    var $sResultDirectoryName = '';
    
    /**
     * Chemin vers le répertoire du résultat de la demande.
     */
    var $sResultDirectoryPath = '';
    
    /**
     * Tableau contenant la liste des fichiers de destination.
     */
    var $aDestinationFiles = array();

    /**
     * Cette méthode permet d'initialiser les variables.
     * \param $oBd Connexion à la base de données.
     * \param $iDemandeId Identifiant de la demande.
     * \param $sRobotLogFile Fichier de log.
     * \param $sParams Liste des paramètres saisis par l'utilisateur à la création de la demande.
     * \param $aProperties Tableau des properties associés à l'application
     * \param $sFmwFileName Nom du fichier FMW associé à la demande.
     */
    function __construct($oBd, $iDemandeId, $sRobotLogFile, $sParams, $aProperties, $sFmwFileName) {
        $this->oBd = $oBd;
        $this->iDemandeId = $iDemandeId;
        $this->sRobotLogFile = $sRobotLogFile;
        $this->sFmwFileName = $sFmwFileName;
        $this->aParametre = $this->dbParamsAsArray($sParams);
        $this->sParams = $sParams;
        $this->aProperties = $aProperties;
        // table de correspondance entre GTF et le FMW utiliser dans les fonction d'encodage 
        $this->aReplaceGTF = array("\r\n", " ", "(", ")", "$", "&", "@", "°", "'", ",", "\"", "{", "}", "[", "]", ";", "/", "\\", "é", "à");
        $this->aReplaceFME = array("<lf>", "<space>", "<openparen>", "<closeparen>", "<dollar>", "<amp>", "<at>", "<u00b0>", "<apos>", "<comma>", "<quote>", "<opencurly>", "<closecurly>", "<openbracket>", "<closebracket>", "<semicolon>", "<solidus>", "<backslash>", "<u00e9>", "<u00e0>");
        //writeToLog(str_replace('[this->sFmwFileName]', $this->sFmwFileName, str_replace('[sParams]', $sParams, INFO_RESOURCES)), $this->sRobotLogFile);
    }

    /**
     * Cette méthode permet de générer un tableau de paramètres à partir d'une chaîne dont les délimiteurs sont "|" et "=".
     * \param $sAllParams Chaine composé de tous les paramètres.
     * \return un tableau contenant les paramètres.
     */
    function dbParamsAsArray($sAllParams) {
        $aAllParams = Array();
        $aLigne = explode("|", $sAllParams);
        //$aLigne = stringExplode($sAllParams, "|", "\"", "\"");
        foreach ($aLigne as $sValeur) {
            $aValeurs = explode("=", $sValeur, 2);
            if ($aValeurs[1] == "") {
                writeToLog(INFO_NULL_ELEMENT . $aValeurs[0] . ".", $this->sRobotLogFile);
            }
            if (!empty($aValeurs[2])) {
                $sTemp = $aValeurs[1] . "=" . $aValeurs[2];
                $aValeurs[1] = $sTemp;
            }
            $aAllParams[$aValeurs[0]] = $aValeurs[1];
            writeToLog(str_replace('[aValeurs[0]]', $aValeurs[0], INFO_DB_PARAM) . $aValeurs[1], $this->sRobotLogFile);
        }
        return $aAllParams;
    }

    /**
     * Cette méthode permet d'éxécuter une ligne de commande et de stocker les paramètres dans un fichier
     * \param $sCommand Ligne de commande à éxécuter.
     */
    function execute($sCommand, $sFMELogFile) {
        $sExeName = "FME";
        if (!$this->bErreur) {
            set_time_limit(0);
            // Déclaration du tableau de résultat (IMPORTANT!).
            $aResult = array();
            //$sResultCommande = exec('"'.$this->aProperties["fme_path"].'" APPLY_SETTINGS "Python/Python Interpreter"', $aResult, $iResult);
            $sResultCommande = exec('"' . $this->aProperties["fme_path"] . '" APPLY_SETTINGS "Python/Use Custom Python" false', $aResult, $iResult);
            //writeToLog(INFO_INFORM.$sExeName."| ".'"'.$this->aProperties["fme_path"].'" APPLY_SETTINGS "Python/Python Interpreter"', $this->sRobotLogFile);
            //writeToLog(INFO_INFORM.$sExeName."| ".$sResultCommande, $this->sRobotLogFile);
            // $sResultCommande = exec('"'.$this->aProperties["fme_path"].'" APPLY_SETTINGS "Python/Python Interpreter" "C:/apps/FME-2015.1-b15477-32/fmepython27/python27.dll"', $aResult, $iResult);
            // writeToLog(INFO_INFORM.$sExeName."| APPLY_SETTINGS \"Python/Python Interpreter\" \"C:/apps/FME-2015.1-b15477-32/fmepython27\"", $this->sRobotLogFile);
            // writeToLog(INFO_INFORM.$sExeName."| ".$sResultCommande, $this->sRobotLogFile);
            $sResultCommande = exec($sCommand, $aResult, $iResult);
            if ($sResultCommande == "") {
                $aResultCommande[0] = true;
            } else {
                $aResultCommande = explode("|", $sResultCommande);
            }
            //Récupération d'une erreur.
            if ($iResult !=0){
            //if ($iResult || ($aResultCommande[0] != 0)) {
                $this->bErreur = true;
                if (sizeOf($aResultCommande) == 1)
                    $this->bFmeCrash = false;
            }
            //Écriture dans le log
            if ($this->bErreur) {
                if (!empty($aResultCommande[1])) {
                    if ($this->aProperties["convert_utf8"]) {
                        $aResultCommande[1] = utf8_decode($aResultCommande[1]);
                    }
                    $this->sMessageErreur = "Erreur lors de l exécution de la commande." . $aResultCommande[1] . ".";
                } else {
                    $this->sMessageErreur = "Erreur lors de l exécution de la commande. Consultez le log du robot.";
                }
                $bExistFMELogFile = true;
                 if (file_exists($sFMELogFile) === false) {
                    $bExistFMELogFile = false;
                }
                foreach ($aResult as $sValeur) {
                    writeToLog(INFO_ERROR . "   " . "| " . $sValeur, $this->sRobotLogFile);
                    if ($bExistFMELogFile === false){
                        $monfichier = fopen($sFMELogFile, 'a');
                        fputs($monfichier, $sValeur. "\n");
                        fclose($monfichier);
                    }
                }
                writeToLog(INFO_COMMAND_ERROR . $sCommand . ".", $this->sRobotLogFile);
            } else {
                foreach ($aResult as $sValeur) {
                    if (mb_detect_encoding($sValeur, "UTF-8, ISO-8859-1") != "UTF-8") {
                        $sValeur = iconv("ISO-8859-1", "UTF-8", $sValeur);
                    }
                    writeToLog(INFO_INFORM . $sExeName . "| " . $sValeur, $this->sRobotLogFile);
                }
                writeToLog(INFO_SUCCESSFUL_TREATMENT, $this->sRobotLogFile);
            }
        }
    }

    /**
     * Cette méthode permet de 7zipper des fichiers.
     * \param $sZipDir Emplacement des fichiers à zipper.
     * \param $sZipUrl URL du fichier zip.
     * \param $sZipFileName Nom du fichier zip à compresser.
     * \return L'url du fichier zip.
     */
    function zipFile($sZipDir, $sZipFileDir, $sZipUrl, $sZipFileName) {
        writeToLog(str_replace('[sZipDir]', $sZipDir, INFO_FILE_COMPRESSION), $this->sRobotLogFile);
        if (createZip($sZipDir, $sZipFileDir . '/' . $sZipFileName . '.zip', 'log', $this->sSecretKey, true)) {
            $sZipFile = $sZipUrl . "/" . $sZipFileName . ".zip";
            writeToLog(INFO_COMPRESSED_FILES . $sZipFile . ".", $this->sRobotLogFile);
        } else {
            $this->bErreur = true;
            $sZipFile = "";
            writeToLog(INFO_TREATMENT_FAILURE, $this->sRobotLogFile);
            writeToLog(str_replace('[sZipFileDir]', $sZipFileDir, str_replace('[sZipDir]', $sZipDir, INFO_FILES_COMPRESSION_FAILURE)) . $sZipFileName, $this->sRobotLogFile);
            $this->sMessageErreur = "Le traitement n\'a produit aucun résultat. Consultez son log.";
        }
        return $sZipFile;
    }

    function unZipSource($sParam) {
        $this->sSource = $sParam;
        // Si la chaine contient un accent, la chaine sera en UTF-8, la fonction file_exists et la fonction de dézippage ne gèrent pas utf-8, il faut donc convertir la chaine en ISO-8859-1
        $bUtf8Param = false;
        if (mb_check_encoding($sParam, 'UTF-8')) {
            $sParam = utf8_decode($sParam);
            $bUtf8Param = true;
        }
        // Si le fichier n'est pas trouvé alors il se trouve dans le répertoire upload
        if (!file_exists($sParam))
            $sParam = $this->aProperties["upload_dir"] . "/" . $sParam;

        // If faut gérer l'ensemble des zip (.zip, .7z, .rar, bzip2, gzip, tar, wim, xz)
        if (mb_eregi("(.ZIP)$", strtoupper($sParam), $aOccurence)) {
            // Décompression du fichier zip.
            $sDir = $this->unZipFile($sParam);
        } else { // Le fichier uploadé n'est pas un fichier zippé
            $sDir = $sParam;
            $this->sSourceTemp = "";
        }
        // Si le paramètre passé était encodé en UTF-8 il faut le réencoder sinon problème avec les accents.
        if ($bUtf8Param)
            $sDir = utf8_encode($sDir);
        return $sDir;
    }

    /**
     * Cette méthode permet de dézipper un fichier.
     * \param $sZipFile Le fichier zip à décompresser.
     * \return L'emplacement des fichiers qui ont été extraits.
     */
    function unZipFile($sZipFile) {
        if (!$this->bErreur) {
            writeToLog(INFO_DECOMPRESSING_FILE . $sZipFile . ".", $this->sRobotLogFile);
            // Les backslashes du chemin de la variable d'environnement TEMP sont remplacés par des slashes
            $this->aProperties["extract_dir"] = str_replace("\\", "/", $this->aProperties["extract_dir"]);
            do {
                $sExtractDir = $this->aProperties["extract_dir"] . "/" . getUniqRandomId() . "/";
            }while(file_exists($sExtractDir));
            $this->sSourceTemp = $sExtractDir;
            if (unZip($sZipFile, $sExtractDir)) {
                writeToLog(INFO_FILE_COMPRESSION_DIRECTORY . $sExtractDir . ".", $this->sRobotLogFile);
            } else {
                $this->bErreur = true;
                writeToLog(INFO_TREATMENT_FAILURE, $this->sRobotLogFile);
                writeToLog(str_replace('[sZipFile]', $sZipFile, INFO_FILE_COMPRESSION_FAILURE) . $sExtractDir . ".", $this->sRobotLogFile);
                $this->sMessageErreur = "Impossible de dézipper le fichier. Consultez le log du robot.";
                $sExtractDir = "";
            }
            return $sExtractDir;
        }
    }

    /*
     * Fonction qui se charge de la mise à jour du paramètre "source" s'il existe
     * $oGtfFmwParser : Objet GtfFmwParser contenant les GUI
     * $oGui : Objet GUI de la source
     * $sDirSource : Chaine de caractère contenant le chemin d'accés à la source de données
     * \return Tableau des fichiers pour le paramètre "source".
     */

    function getSourceParam($oGui, $oGtfFmwParser, $sDirSource, $bUsePattern) {
        $aSourceMacro = array();
        $i = 0;
        $aSourceFiles = array();
        if (is_dir($sDirSource)) {
            $this->aExtension = $oGui->getExtensionArray($oGui->sFiltreSource);
            $iExtension = count($this->aExtension);
            if (($this->aExtension[0] == "*.*" and $oGui->sType == "FILE_OR_URL") or $oGui->sType == "DIRNAME_SRC" or $oGui->sType == "SOURCE_GEODATABASE") {
                writeToLog(INFO_VALID_DATA_SOURCE_FILE_EXTENSION . $this->aExtension[0], $this->sRobotLogFile);
                $aSourceFiles[] = $sDirSource . '/**/';
            } else {
                if ($bUsePattern){
                        writeToLog(INFO_USE_PATTERN, $this->sRobotLogFile);
                        foreach ($this->aExtension as $sExtension) {
                                if ($sExtension != "*.*" or ( $sExtension == "*.*" and $iExtension == 1)) { // Si le filtre est différent de *.* (ex : *.mdb) ou si le filtre est seulement *.*. Si le filtre est multiple et que l'un d'eux est *.* (ex : *.*, *.mdb...), il ne sera pas pris en compte
                                        $aExtension = explode(".", $sExtension);
                                        if ($this->countFilesExtension($sDirSource, $aExtension[1]) > 0) {
                                                writeToLog(INFO_VALID_DATA_SOURCE_FILE_EXTENSION . $sExtension, $this->sRobotLogFile);
                                                $aSourceFiles[] = $sDirSource . '/**/' . $sExtension;
                                                $i++;
                                        }
                                }
                        }
                }else{
                        writeToLog(INFO_NO_USE_PATTERN, $this->sRobotLogFile);
                        foreach ($this->aExtension as $sExtension) {
                                if ($sExtension != "*.*" or ( $sExtension == "*.*" and $iExtension == 1)) { // Si le filtre est différent de *.* (ex : *.mdb) ou si le filtre est seulement *.*. Si le 
                                        $aExtension = explode(".", $sExtension);
                                        $aSourceMacroTmp = array_merge($aSourceMacro, $this->returnFileListToString($sDirSource, $aExtension[1]));
                                        $aSourceMacro = array_unique($aSourceMacroTmp);
                                        $i++;
                                }
                        }
                        foreach ($aSourceMacro as $source) {
                                $aSourceFiles[] = $source;
                        }
                }
            }
        }else
            $aSourceFiles[] = $sDirSource;
        
        return $aSourceFiles;
    }

    // Compte le nombre de fichier
    // $sDirSource : Répertoire de base
    // $sExtension : Filtre
    function countFilesExtension($sDirSource, $sExtension) {
        $num = 0;
        if (!is_dir($sDirSource)) {
            $path_parts = pathinfo($sDirSource);
            if (is_file($sDirSource) && $path_parts['extension'] == $sExtension)
                return 1;
            else
                return false;
        }
        foreach (scandir($sDirSource) AS $entry) {
            if (!in_array($entry, array('..', '.')))
                $num += $this->countFilesExtension($sDirSource . "/" . $entry, $sExtension);
        }
        return $num;
    }

    function returnFileListToString($sDirSource, $sExtension) {
    $aFileList = array();
    if (!is_dir($sDirSource)) {
        $path_parts = pathinfo($sDirSource);
        if (is_file($sDirSource) && strtolower($path_parts['extension']) == strtolower($sExtension)){
            array_push($aFileList,$sDirSource);
        }

    }else{
            foreach (scandir($sDirSource) AS $entry) {
                    if (!in_array($entry, array('..', '.'))){
                            $aFileListTmp = $aFileList;
                            $aFileList = array_merge($aFileListTmp,$this->returnFileListToString($sDirSource . "/" . $entry, $sExtension));
                    }

            }
        }
        return $aFileList;
    }

    /*
     * Fonction qui se charge de la mise à jour du paramètre "source" s'il existe
     * $oGtfFmwParser : Objet GtfFmwParser contenant les GUI
     * $oGui : Objet GUI de la source
     * $sDirSource : Chaine de caractère contenant le chemin d'accés à la source de données
     */

    function updateDestParam($oGui, $oGtfFmwParser, $sDest) {
        $this->aExtension = $oGui->getExtensionArray($oGui->sFiltreDest);
        if ($sDest == "" && $oGui->sType != "DIRNAME")
            $sDest = $this->sResultDirectoryName;
        $this->aParametre[$oGui->sDefault_Macro] = "";
        $sDest = normalizeString($sDest);
        $aDest = explode(".", $sDest);
        if ($oGui->sType == "DIRNAME" or $oGui->sType == "DEST_GEODATABASE") {
            $this->aParametre[$oGui->sDefault_Macro] = $this->aProperties["dir_export"] . "/gtf/" . $this->sResultDirectoryName . "/" . $sDest;
            writeToLog(INFO_OUTPUT_DATA_DIRECTORY . $this->aParametre[$oGui->sDefault_Macro], $this->sRobotLogFile);
        }
        if ($oGui->sType == "FILENAME") {
            if (count($aDest) > 1) { // Dans GTF, la valeur du champ [nom de fichier en sortie] comporte une extension. Ex : result.xls (le fichier résultat sera de l'excel et sera nommé result.
                $sResult = $sDest;
            } else {
                $sResult = str_replace("*", $sDest, $this->aExtension[0]);
            }
            $this->aParametre[$oGui->sDefault_Macro].= $this->aProperties["dir_export"] . "/gtf/" . $this->sResultDirectoryName . '/' . $sResult;
            $this->sDestination = $sResult;

            writeToLog(INFO_OUTPUT_DATA_FILE . $this->aParametre[$oGui->sDefault_Macro], $this->sRobotLogFile);
        }
        else if ($oGui->sType == "FILENAME_EXISTING")
            $this->aParametre[$oGui->sDefault_Macro] = $sDest;
        return $this->aParametre[$oGtfFmwParser->sDestMacro];
    }

    /*     * *************************************************************************************** */
    /*                                          Codage                                        */

    function unichr($u) {
        return mb_convert_encoding('&#' . intval($u) . ';', 'UTF-8', 'HTML-ENTITIES');
    }

    function ordutf8($string, &$offset) {
        $code = ord(substr($string, $offset, 1));
        if ($code >= 128) {        //otherwise 0xxxxxxx
            if ($code < 224)
                $bytesnumber = 2;                //110xxxxx
            else if ($code < 240)
                $bytesnumber = 3;        //1110xxxx
            else if ($code < 248)
                $bytesnumber = 4;    //11110xxx
            $codetemp = $code - 192 - ($bytesnumber > 2 ? 32 : 0) - ($bytesnumber > 3 ? 16 : 0);
            for ($i = 2; $i <= $bytesnumber; $i++) {
                $offset ++;
                $code2 = ord(substr($string, $offset, 1)) - 128;        //10xxxxxx
                $codetemp = $codetemp * 64 + $code2;
            }
            $code = $codetemp;
        }
        $offset += 1;
        if ($offset >= strlen($string))
            $offset = -1;
        return $code;
    }

    function encoderFME($sStr) {
        //table de correcpondance des symboles UTF-8 avec les codes FME/GTF
        // ATTENTION : laisser < > en premier sinon replace remplace les balises par d'autre balise
        $aReplaceGTFToFMW = array("<" => "<lt>", ">" => "<gt>");
        $sReplaced = $sStr;

        //on gere les chevrons simultanèment pour ne pas gener l'encodage après
        if (strpos($sReplaced, "<") !== false && strpos($sReplaced, ">") !== false) {
            $sReplaced = strtr($sReplaced, $aReplaceGTFToFMW);
        }

        //on remplace tous les encodage par leurs symboles
        for ($i = 0; $i < count($this->aReplaceGTF); $i++) {
            if (strpos($sReplaced, $this->aReplaceGTF[$i]) !== false) {
                $sReplaced = str_replace($this->aReplaceGTF[$i], $this->aReplaceFME[$i], $sReplaced);
            }
        }

        // on encode les balises UTF-8      

        $offset = 0;
        $iPos = 0;

        while ($offset >= 0) {
            $iCode = $this->ordutf8($sReplaced, $offset);
            if ($iCode > 127) {
                $cLetter = dechex($iCode);
                while (strlen($sLetter) < 4) {
                    $cLetter = "0" . $cLetter;
                }
                $sReplaced = str_replace($sReplaced[$iPos], "<u" . $cLetter . ">", $sReplaced);
            }
            $iPos += 1;
        }

        return $sReplaced;
    }

    function decoderFME($sStr) {
        //table de correcpondance des symboles UTF-8 avec les codes FME/GTF
        // ATTENTION : laisser < > en premier sinon replace remplace les balises par d'autre balise
        $aReplaceFMWToGTF = array("<lt>" => "<", "<gt>" => ">");
        $sReplaced = $sStr;

        //on gere les chevrons simultanèment pour ne pas gener l'encodage après
        if (strpos($sReplaced, "<lt>") !== false && strpos($sReplaced, "<gt>") !== false) {
            $sReplaced = strtr($sReplaced, $aReplaceFMWToGTF);
        }
        //on remplace tous les encodage par leurs symboles
        for ($i = 0; $i < count($this->aReplaceGTF); $i++) {
            if (strpos($sReplaced, $this->aReplaceFME[$i]) !== false) {
                $sReplaced = str_replace($this->aReplaceFME[$i], $this->aReplaceGTF[$i], $sReplaced);
            }
        }

        // on decode les balises UTF-8      
        $matches = array();

        preg_match_all("/<u[0-9a-fA-F]{4}>/", $sStr, $matches);

        for ($i = 0; $i < count($matches[0]); $i++) {
            $iCode = hexdec(substr($matches[0][$i], 2, 4));
            $cChr = $this->unichr($iCode);
            $sReplaced = str_replace($matches[0][$i], $cChr, $sReplaced);
        }
        return $sReplaced;
    }

    function isDecode($sStr) {
        for ($i = 0; $i < count($this->aReplaceFME); $i++) {
            if (strpos($sStr, $this->aReplaceFME[$i]) !== false && strpos($sStr, $this->aReplaceGTF[$i]) === false) {
                return false;
            }
        }
        return true;
    }

    function isEncode($sStr) {
        for ($i = 0; $i < count($this->aReplaceFME); $i++) {
            if (strpos($sStr, $this->aReplaceFME[$i]) === false && strpos($sStr, $this->aReplaceGTF[$i]) !== false) {
                return false;
            }
        }
        return true;
    }

    /*     * *************************************************************************************** */

    /*
     *  Fonction qui permet de concaténer tous les paramètres nécessaire pour l'appel du TCL
     *  et qui retourne $sChaine, la chaine des paramètres à passer au TCL
     *  $oGtfFmwParser : Objet GtfFmwParser contenant les GUI
     */

    function getTclParams($oGtfFmwParser) {
        $sChaine = "";
        $aFmeParameters = $this->getFmeParameters($oGtfFmwParser);
        foreach ($aFmeParameters as $sFmeParamKey => $sFmeParamValue) {
            // Echappement des doubles quotes pour FME.
            $sFmeParamValue = str_replace('"', '\"', $sFmeParamValue);
            // Ajout du paramètre en ligne de commande.
            $sChaine .= " --" . $sFmeParamKey . " \"" . $sFmeParamValue . "\"";
        }
        // $sChaine = substr($sChaine, 0, -1)."\"";
        //$sChaine = substr($sChaine, 0, -1);
        return $sChaine;
    }

    function listDir($iDemandeId) {
        $ouverture = @opendir($this->sResultDirectoryPath);
        if (!$ouverture)
            return false;
        while ($sFichier = readdir($ouverture)) {
            if ($sFichier == '.' || $sFichier == '..')
                continue;
            if (is_dir($this->sResultDirectoryPath . $sFichier)) {
                closedir($ouverture);
                return true;
            } else {
                $path_parts = pathinfo($this->sResultDirectoryPath . $sFichier);
                if ($sFichier != "fme_" . $iDemandeId . ".log") {
                    closedir($ouverture);
                    return true;
                }
            }
        }
        closedir($ouverture);
        return False;
    }

    /*
     * Fonction qui retourne le nom du fichier résultat qui sera zippé.
     *  $iDemandeId correspond au numéro de la demande
     * \return Retourne NULL (s'il y a plusieurs fichiers résultat à zipper) ou le fichier résultat qui sera zippé sous la forme 20140324141501876/ventes.xlsx
     */

    function CheckResult($iDemandeId) {
        $ouverture = @opendir($this->sResultDirectoryPath);
        $NbFile = 0;
        while ($sFichier = readdir($ouverture)) {
            if ($sFichier == '.' || $sFichier == '..')
                continue;
            if (is_dir($this->sResultDirectoryPath . $sFichier)) {
                $NbFile = $NbFile + 2;
            } else {
                $path_parts = pathinfo($this->sResultDirectoryPath . $sFichier);
                if ($sFichier != "fme_" . $iDemandeId . ".log") {
                    $NbFile = $NbFile + 1;
                }
            }
        }
        closedir($ouverture);
        $ouverture = @opendir($this->sResultDirectoryPath);
        if ($NbFile != 1) {
            return "";
        } else {
            $pattern = '/^' . str_replace("*", ".+", str_replace(".", "\\.", $this->aProperties['not_compressed_extension'])) . '$/';
            while ($sFichier = readdir($ouverture)) {
                if ($sFichier == '.' || $sFichier == '..')
                    continue;
                if ($sFichier != "fme_" . $iDemandeId . ".log") {
                    if (preg_match($pattern, $sFichier) && !$this->bDataEncrypt) {
                        return $this->sResultDirectoryName . "/" . $sFichier;
                    } else {
                        return "";
                    }
                }
            }
        }
        closedir($ouverture);
    }

    /*
     * fonction qui lance la génération de la ligne de commande, l'éxécution de la demande et la mise en forme du résultat
     * \return Retourne les fichiers résultant zippés
     */

    function Process() {
        $oGtfFmwParser = new GtfFmwParser($this->aProperties["workspace_dir"] . "/" . $this->sFmwFileName, $this->aProperties);
        // Création du répertoire de résultat.
        $this->sResultDirectoryName = $this->createResultDirectory();

        //1 Initialisation & calcul du chemin d'accés du fichier TCL
        $sCommandLine = '"' . $this->aProperties["fme_path"] . '" "' . getenv("GTF_HOME") . "/" . utf8_decode($this->sFmwFileName) . '"';

        //3 Ajout des paramètres propres au traitement
        $sTclParams = $this->getTclParams($oGtfFmwParser);
        if ($sTclParams != "")
            $sCommandLine .= $sTclParams;
        $aTGui = array();
        $sChaine = "";
        
        // Ajout des paramètres GTF_ s'ils sont associés à un paramètre publié du traitement
        $aGtfParameters = $this->getGtfParameters($oGtfFmwParser);
        foreach ($aGtfParameters as $sGtfParamKey => $sGtfParamValue) {
            $sCommandLine .= ' --' . $sGtfParamKey . ' "' . $sGtfParamValue . '"';
        }
        /*
         * 2 Passage des paramètres généraux
         * - FMWFILENAME : fichier Fmw
         * - LOG_ROBOT : fichier log du robot
         * - LOG_FILENAME : fichier log pour FME
         */
        $sCommandLine.= " -LOG_FILENAME \"" . $this->getFmeLogFilePath($this->sResultDirectoryPath) . "\" 2>&1";
        writeToLog(INFO_GTF_HOME . getenv("GTF_HOME"), $this->sRobotLogFile);
        writeToLog(INFO_FME_PATH . getenv("FME_PATH"), $this->sRobotLogFile);
        writeToLog(INFO_TREATMENT_COMMAND_LINE . $sCommandLine, $this->sRobotLogFile);

        //4 Lancement de la ligne de commande
        $this->sLogFme = $this->getFmeLogFilePath($this->sResultDirectoryName);
        $sSql = "UPDATE " . $this->aProperties['schema_gtf'] . ".order SET log_url='" . $this->sLogFme . "' WHERE order_id=" . $this->iDemandeId;
        $this->oBd->execute($sSql);
        $this->execute($sCommandLine, $this->getFmeLogFilePath($this->sResultDirectoryPath));
        writeToLog(INFO_TREATMENT_COMMAND_LINE . $sCommandLine, $this->sRobotLogFile);
        if (file_exists($this->getFmeLogFilePath($this->sResultDirectoryPath)) == false) {
            $this->sLogFme = "";
        }
        // Tous les fichiers de résultat dans une archive .zip.
        $sZipFile = $this->zipResultFiles();
        return $sZipFile;
    }
    
    /**
     * Submit a job on Fme Server.
     * @param {object} $oFmeServer FmeServer class object.
     * @param {array} $aOrder Order parameters.
     * @param {array} $aWorkspace Workpace parameters.
     * @param {array} $aUser User infos.
     * @param {object} $oAmazonS3 AmazonS3 class object.
     * @param {boolean} $bAsync Asynchronous job.
     * @return string
     */
    function submitFmeServerJob($oFmeServer, $aOrder, $aWorkspace, $aUser, $oAmazonS3 = null, $bAsync = false) {
        $this->oFmeServer = $oFmeServer;
        $this->oAmazonS3 = $oAmazonS3;
        // Création du répertoire de résultat.
        $this->sResultDirectoryName = $this->createResultDirectory();
        // Fichiers sources en entrée et sortie.
        $oGtfFmwParser = new GtfFmwParser($this->aProperties["workspace_dir"] . "/" . $this->sFmwFileName, $this->aProperties);
        $aTGui = array();
        $aSourceFiles = array();
        $aDestinationFiles = array();
        foreach ($oGtfFmwParser->aGuiObject as $oGui) {
            if ($oGui->sDefault_Macro != "") {
                array_push($aTGui, $oGui);
                //Teste si le paramètre est une source
                if ($oGui->bIsSource)
                    $aSourceFiles[$oGui->sDefault_Macro] = $this->aParametre[$oGui->sDefault_Macro];
                else if ($oGui->bIsDest)
                    $aDestinationFiles[] = $oGui->sDefault_Macro;
            }
        }
        // Sauve la liste des fichiers de destination.
        $this->aDestinationFiles = $aDestinationFiles;
        // Liste des paramètres publiés.
        $aFmeParameters = $this->getFmeParameters($oGtfFmwParser);
        // Ajout des paramètres GTF_ s'ils sont associés à un paramètre publié du traitement.
        $aFmeParameters = array_merge($aFmeParameters, $this->getGtfParameters($oGtfFmwParser));
        // Création d'un identifiant pour l'instance de GTF (si inexistant).
        if (empty($this->aProperties["gtf_instance_id"]))
            $this->aProperties["gtf_instance_id"] = generateGtfInstanceId();
        // Création d'un dépot pour l'instance de GTF si inexistant.
        $sRepository = $this->aProperties["gtf_instance_id"];
        if ($oFmeServer->createRepository($sRepository, '') === false)
            return false;
        // Création d'un répertoire dans les ressources si inexistant.
        $sResource = 'FME_SHAREDRESOURCE_TEMP';
        $sPath = $this->aProperties["gtf_instance_id"];
        if ($oFmeServer->createResourceConnectionDirectory($sResource, $sPath, $this->sResultDirectoryName) === false)
            return false;
        $sPath .= '/' . $this->sResultDirectoryName;
        // Copie locale du fichier .fmw avec la version.
        $sFmwFilePath = $this->aProperties["workspace_dir"] . "/" . $this->sFmwFileName;
        $sRepositoryFmwFileName = pathinfo($this->sFmwFileName, PATHINFO_FILENAME) . '_v' . $aWorkspace['version'] . '.' . pathinfo($this->sFmwFileName, PATHINFO_EXTENSION);
        $sRepositoryFmwFilePath = $this->aProperties["workspace_dir"] . "/" . pathinfo($this->sFmwFileName, PATHINFO_DIRNAME) . "/" . $sRepositoryFmwFileName;
        if (!file_exists($sRepositoryFmwFilePath)) {
            copy($sFmwFilePath, $sRepositoryFmwFilePath);
            // Supprime le fichier de l'ancienne version du projet (GTF + Fme Server).
            if ($aWorkspace['version'] > 0) {
                $sOldFmwFileName = pathinfo($this->sFmwFileName, PATHINFO_FILENAME) . '_v' . --$aWorkspace['version'] . '.' . pathinfo($this->sFmwFileName, PATHINFO_EXTENSION);
                $sOldFmwFilePath = $this->aProperties["workspace_dir"] . "/" . pathinfo($this->sFmwFileName, PATHINFO_DIRNAME) . "/" . $sOldFmwFileName;
                if (file_exists($sOldFmwFilePath))
                    unlink($sOldFmwFilePath);
                //$oFmeServer->serviceRequest('repositories/' . $sRepository . '/items/' . $sOldFmwFileName, 'delete');
                // S3 -> copie du fichier avec la dernière version et suppression de l'ancien.
                if ($this->aProperties['fileS3Uploader'] === true) {
                    deleteFileFromWsDataDirTree('gtf', 'workspace', $aWorkspace['workspace_id'], 'fme', $sOldFmwFileName);
                    copyFileInWsDataDir ('gtf', 'workspace', $aWorkspace['workspace_id'], 'fme', pathinfo($this->sFmwFileName, PATHINFO_BASENAME), $sRepositoryFmwFileName);
                }
            }
        }
        // Upload du fichier .fmw.
        if ($oFmeServer->uploadWorkspaceFile($sFmwFilePath, $sRepository, false, $sRepositoryFmwFileName) === false)
            return false;
        // Upload des fichiers sources.
        foreach($aSourceFiles as $sParamName => $sParamValue) {
            if ($this->uploadFileToResourceConnection($this->aProperties["upload_dir"] . "/" . $sParamValue, $sRepository, $sResource, $sPath) === false)
                return false;
            /*
            // Upload de plusieurs fichiers sources (ex: contenu d'une archive .zip).
            $aFiles = explode('|', $aFmeParameters[$sParamName]);
            foreach ($aFiles as $sFilePath) {
                if ($this->uploadFileToResourceConnection($sFilePath, $sRepository, $sResource, $sPath) === false)
                    return false;
            }
            */
        }
        // Upload des ressources complémentaires du projet.
        $sResourcesDirectory = $this->aProperties["workspace_dir"] . '/' . pathinfo($this->sFmwFileName, PATHINFO_DIRNAME);
        if (is_dir($sResourcesDirectory)) {
            foreach (scandir($sResourcesDirectory) as $sFileName) {
                if (is_file($sResourcesDirectory . '/' . $sFileName) && array_search(pathinfo($sFileName, PATHINFO_EXTENSION), array('fmw', 'bak')) === false) {
                    if ($oFmeServer->uploadWorkspaceResourceFile($sResourcesDirectory . '/' . $sFileName, $sRepository, $sRepositoryFmwFileName, false) === false)
                        return false;
                }
            }
        }
        // Paramètres publiés dans les données de la requête.
        $aJobParams = array('publishedParameters' => array());
        // Traitement asynchrone.
        if ($bAsync) {
            $aJobParams['NMDirectives'] = array(
                'successTopics' => array(
                    'TEST_TOPIC_FRED'
                ),
                'failureTopics' => array(
                    'TEST_TOPIC_FRED'
                ),
                'directives' => array(
                    array('name' => 'order_id', 'value' => $this->iDemandeId)
                )
                //'subsection' => 'REST_SERVICE'
            );
        }
        // Chemins pour les paramètres publiés qui sont des fichiers.
        foreach ($aFmeParameters as $sParamName => $sParamValue) {
            if (array_key_exists($sParamName, $aSourceFiles)) {
                $aFiles = explode('|', $sParamValue);
                $aPublishedParametersFiles = array();
                foreach ($aFiles as $sFilePath)
                    $aPublishedParametersFiles[] = '$(' . $sResource . ')/' . $sPath . '/' . $sFilePath;
                $aJobParams['publishedParameters'][] = array("name" => $sParamName, "value" => $aPublishedParametersFiles);
            }
            else if (in_array($sParamName, $aDestinationFiles))
                $aJobParams['publishedParameters'][] = array("name" => $sParamName, "value" => '$(' . $sResource . ')/' . $sPath . '/' . $this->sResultDirectoryName . '.zip/' . pathinfo($sParamValue, PATHINFO_BASENAME));
            else
                $aJobParams['publishedParameters'][] = array("name" => $sParamName, "value" => $sParamValue);
        }
        // Envoi d'une demande sur un traitement.
        $this->oJobResult = $oFmeServer->submitJob($sRepository, $sRepositoryFmwFileName, $aJobParams, $bAsync);
        if ($this->oJobResult === false)
            return false;
        // Demande synchrone -> téléchargement des fichiers de log et de résultat du traitement.
        if ($bAsync) {
            $this->aJobDestinationFiles = $aDestinationFiles;
            $this->sJobResource = $sResource;
            $this->sJobPath = $sPath;
        }
        else {
            $sZipFile = '';
            if (!empty($this->oJobResult->id))
                $sZipFile = $this->downloadFmeServerJobResult($this->oJobResult->id, $oFmeServer, $aDestinationFiles, $sResource, $sPath);
            return $sZipFile;
        }
    }
    
    /**
     * Init all the encryption parameters of the user.
     */
    function initUserEncryptionParameters() {
        // Paramètres de cryptage de l'utilisateur.
        $sSql = 'SELECT "order".user_id, workspace.workspace_id, "user".login, "user".dataencrypt AS user_dataencrypt, workspace.dataencrypt AS workspace_dataencrypt, "user".secretkey FROM [sSchemaGtf].order LEFT JOIN [sSchemaFramework].user ON "order".user_id = "user".user_id LEFT JOIN [sSchemaGtf].workspace ON "order".workspace_id = workspace.workspace_id WHERE order_id = [iDemandeId]';
        $aParams = array();
        $aParams['sSchemaFramework'] = array('value' => $this->aProperties["schema_framework"], 'type' => 'schema_name');
        $aParams['sSchemaGtf'] = array('value' => $this->aProperties["schema_gtf"], 'type' => 'schema_name');
        $aParams['iDemandeId'] = array('value' => $this->iDemandeId, 'type' => 'number');
        $oPDOResult = $this->oBd->executeWithParams($sSql, $aParams);
        if (!$this->oBd->enErreur()) {
            $aUserEncryptParams = $this->oBd->ligneSuivante($oPDOResult);
            if ($aUserEncryptParams['user_dataencrypt'] === true || $aUserEncryptParams['workspace_dataencrypt'] === true) {
                // Le résultat de la demande sera crypté.
                $this->bDataEncrypt = true;
                // Clé secrète de l'utilisateur (sinon il faut la générer).
                if ($aUserEncryptParams['user_dataencrypt'] === true && !empty($aUserEncryptParams['secretkey']))
                    // Décryptage de la clé secrète (suppression des caractères "NULL").
                    $this->sSecretKey = str_replace(chr(0), '', des($aUserEncryptParams['login'], hexToString($aUserEncryptParams['secretkey']), 0, 0));
                else {
                    // Génération de la clé secrète.
                    $this->sSecretKey = getUniqRandomId();
                    // Envoi du mail avec la clé secrète générée.
                    $aObjects = array();
                    $aObjects['oOrder'] = new OrderLib($this->oBd, $this->iDemandeId, $this->aProperties, "v_order");
                    $aObjects['oOrder']->formatOrderEmail();
                    $aObjects['oOrder']->sSecretKey = $this->sSecretKey;
                    $oEmail = new Email($this->oBd, -1, $this->aProperties, $aObjects);
                    if (!empty($oEmail->oEmailTemplate->name))
                        $oEmail->send();
                }
            }
        }
        else
            writeToLog(INFO_USER_ENCRYPT_PARAMETERS . "(SQL : $sSql)", $this->sRobotLogFile);
    }
    
    /**
     * Upload a file to a resource connection on Fme Server.
     * @param {string} $sFilePath Path of the file to upload.
     * @param {string} $sRepository Name of the repository.
     * @param {string} $sResource Name of a resource connection.
     * @param {string} $sPath Path, relative to the resource connection.
     */
    function uploadFileToResourceConnection($sFilePath, $sRepository, $sResource, $sPath) {
        // Si un objet de la classe AmazonS3 existe, les ressources seront envoyées sur S3 puis téléchargéss dans FME Server.
        if (empty($this->oAmazonS3)) {
            if ($this->oFmeServer->uploadResourceFile($sFilePath, $sResource, $sPath, true, true) === false)
                return false;
        }
        else {
            // Upload d'un fichier source vers S3.
            //$sS3Bucket = $this->aProperties['fme_cloud_s3_bucket'];
            $sS3ObjectKey = $sPath . '/' . pathinfo($sFilePath, PATHINFO_BASENAME);
            if ($this->oAmazonS3->uploadFile($this->sFmeServerS3Bucket, $sS3ObjectKey, $sFilePath) === false)
                return false;
            // Upload du fichier S3Downloader.fmw (si inexistant).
            if ($this->oFmeServer->uploadWorkspaceFile($this->aProperties['workspace_dir'] . '/S3Downloader.fmw', $sRepository, false) === false)
                return false;
            // Téléchargement depuis FME Server du fichier envoyé sur S3.
            $aS3Params = array(
                'access_key_id' => $this->oAmazonS3->sAwsAccessKeyId,
                'secret_access_key' => $this->oAmazonS3->sAwsSecretAccessKey,
                'bucket_name' => $this->sFmeServerS3Bucket,
                'object_key' => $sS3ObjectKey
            );
            $oJobResult = $this->oFmeServer->uploadResourceFileFromS3($aS3Params, $sRepository, $sResource, $sPath);
            if ($oJobResult === false)
                return false;
        }
    }

    function setUseExternalDbConnection ($bValue){
        if(is_bool($bValue)){
            $this->bUseExternalDbConnection = $bValue;
        }
    }
    
    /**
     * get GTF parameters.
     * @param {object} $oGtfFmwParser All the GUI.
     * @return array
     */
    function getGtfParameters($oGtfFmwParser) {
        // Ajout des paramètres GTF_ s'ils sont associés à un paramètre publié du traitement.
        $aGtfParams = array();
        foreach ($oGtfFmwParser->aGuiObject as $aGui) {
            switch($aGui->sDefault_Macro) {
                 case 'GTF_ORDER_ID':
                    $aGtfParams['GTF_ORDER_ID'] = $this->iDemandeId;
                    break;
                 case 'GTF_SHARED_DIR':
                    $aGtfParams['GTF_SHARED_DIR'] = $this->aProperties["shared_dir"];
                    break;
                 case 'GTF_CONNECTION_STRING':
                    if ($aGui->sType == "PASSWORD") {
                        $sServer = $this->aProperties["server"];
                        $iPort = $this->aProperties["port"];
                        if ($this->bUseExternalDbConnection){
                            $sServer = $this->aProperties["connectionStringExternalIp"];
                            $iPort = $this->aProperties["connectionStringExternalPort"];
                        }
                        $aGtfParams['GTF_CONNECTION_STRING'] = $sServer . "," . $iPort . "," . $this->aProperties["database"] . "," . $this->aProperties["login_scheduler"] . "," . $this->aProperties["password_scheduler"];
                    }
                    else
                        writeToLog(INFO_GTF_CONNECTION_STRING_TYPE_PASSWORD, $this->sRobotLogFile);
                    break;
                 case 'GTF_REST_URL':
                    $aGtfParams['GTF_REST_URL'] = $this->aProperties["web_server_name"] . "/" . $this->aProperties["services_alias"] . '/';
                    break;
                 case 'GTF_USER_RESTRICTION':
                    // Restriction de l'utilisateur.
                    $sSql = 'SELECT "user".restriction FROM [sSchemaGtf].order LEFT JOIN [sSchemaFramework].user ON "order".user_id = "user".user_id WHERE order_id = [iDemandeId]';
                    $aParams = array();
                    $aParams['sSchemaFramework'] = array('value' => $this->aProperties["schema_framework"], 'type' => 'schema_name');
                    $aParams['sSchemaGtf'] = array('value' => $this->aProperties["schema_gtf"], 'type' => 'schema_name');
                    $aParams['iDemandeId'] = array('value' => $this->iDemandeId, 'type' => 'number');
                    $oPDOResult = $this->oBd->executeWithParams($sSql, $aParams);
                    if (!$this->oBd->enErreur()) {
                        $aUserRestriction = $this->oBd->ligneSuivante($oPDOResult);
                        $aGtfParams['GTF_USER_RESTRICTION'] = $aUserRestriction['restriction'];
                    }
                    else
                        writeToLog(INFO_USER_RESTRICTION . "(SQL : $sSql)", $this->sRobotLogFile);
                    break;
            }
        }
        return $aGtfParams;
    }
    
    /**
     * format a GUI occurrence (GtfGui).
     * @param {object} $oGui GUI occurrence.
     * @return string
     */
    function formatGuiOccurrence($oGui) {
        $aBaliseToReplace = array('<GTF_EQUAL>', '<GTF_PIPE>');
        $aReplacedBalise = array('=', '|');
        $sGTFOutput = str_replace($aBaliseToReplace, $aReplacedBalise, $this->aParametre[$oGui->sDefault_Macro]);
        switch ($oGui->sType) {
            case "FLOAT":
                $sGTFOutput = str_replace(",", ".", $sGTFOutput);
                if (!$this->isDecode($sGTFOutput))
                    $sGTFOutput = $this->decoderFME($sGTFOutput);
                break;
            case "LOOKUP_CHOICE":
                if (!$this->isEncode($sGTFOutput))
                    $sGTFOutput = $this->encoderFME($sGTFOutput);
                break;
            case "TEXT_EDIT" :
            case "TEXT_EDIT_OR_NUM" :
                if (!$this->isEncode($sGTFOutput)) {
                    //$sGTFOutput = $this->decoderFME($sGTFOutput);
                    $sGTFOutput = $this->encoderFME($sGTFOutput);
                }
                break;
            case "CHOICE":
            case "LOOKUP_LISTBOX":
                $sGTFOutput = str_replace('"', '""', $sGTFOutput);
                if (!$this->isDecode($sGTFOutput))
                    $sGTFOutput = $this->decoderFME($sGTFOutput);
                break;
            case "LISTBOX":
                if ($this->bFmeServer)
                    $sGTFOutput = explode('<space>', $sGTFOutput);
                else if (!$this->isDecode($sGTFOutput))
                    $sGTFOutput = $this->decoderFME($sGTFOutput);
                break;
            default :
                //$sGTFOutput = str_replace('"', '""', $sGTFOutput);
                if (!$this->isDecode($sGTFOutput))
                    $sGTFOutput = $this->decoderFME($sGTFOutput);
                break;
        }
        return $sGTFOutput;
    }
    
    /**
     * Compress all the result files to a zip archive.
     * @return string
     */
    function zipResultFiles() {
        // Tous les fichiers de résultat dans une archive .zip.
        $bHasFile = $this->listDir($this->iDemandeId);
        if ($this->sSourceTemp != "" && $this->aProperties["debug_mode"] == false)
            clearDir($this->sSourceTemp);
        if ($bHasFile) {
            // Paramètres de cryptage de l'utilisateur.
            $this->initUserEncryptionParameters();
            //
            $sIsToZip = $this->CheckResult($this->iDemandeId);
            $sDestUrl = $this->aProperties["url_export"] . "/gtf/" . $this->sResultDirectoryName;
            if ($sIsToZip == "") {
                //Compression en zip des fichiers résultant. Le parametres Array() permet de filtrer les extensions à compresser. Exemples : Array("shp","shx","dbf") ; Array("*")      
                $sZipFile = $this->zipFile($this->sResultDirectoryPath, $this->sResultDirectoryPath, $sDestUrl, $this->iDemandeId);
                $sZipFile = str_replace($sDestUrl, $this->sResultDirectoryName, $sZipFile);
            }
            else
                $sZipFile = str_replace($sDestUrl, $this->sResultDirectoryName, $sIsToZip);
        }
        else
            $sZipFile = "";
        return $sZipFile;
    }
    
    /**
     * Create a directory for the result files.
     * @return string
     */
    function createResultDirectory() {
        // Création du répertoire de sortie des données
        if (!file_exists($this->aProperties["dir_export"] . '/gtf/')){
            mkdir($this->aProperties["dir_export"] . '/gtf/');
        }
        Do {
            $sResultDirectoryName = getUniqRandomId();
            $this->sResultDirectoryPath = $this->aProperties["dir_export"] . '/gtf/' . $sResultDirectoryName;
        }while(file_exists($this->sResultDirectoryPath));
        // Création d'un répertoire pour le fichier de log FME.
        mkdir($this->sResultDirectoryPath . '/log', 0777, true);
        return $sResultDirectoryName;
    }
    
    /**
     * get GTF parameters.
     * @param {object} $oGtfFmwParser All the GUI.
     * @return array
     */
    function getFmeParameters($oGtfFmwParser) {
        $aTGui = array();
        $aFmeParameters = array();
        $bUsePattern = false;
        foreach ($oGtfFmwParser->aGuiObject as $aGui) {
            if ($aGui->sDefault_Macro != "") {
                array_push($aTGui, $aGui);
            }
            if ($aGui->sDefault_Macro == "GTF_FORCE_PATTERN") {
                if (strtolower($aGui->sDefaultValue) == "true"){
                        $bUsePattern = true;
                }
            }
        }
        foreach ($aTGui as $oGui) {
            //Teste si le paramètre est une source
            if ($oGui->bIsSource) {
                if ($this->aParametre[$oGui->sDefault_Macro] != "" && substr($this->aParametre[$oGui->sDefault_Macro], 0, 7) != "<quote>") {
                    $this->aParametre[$oGui->sDefault_Macro] = str_replace('//', '/', $this->aParametre[$oGui->sDefault_Macro]);
                    $sSourceFilePath = $this->aParametre[$oGui->sDefault_Macro];
                    $sDirSource = $this->unZipSource($this->aParametre[$oGui->sDefault_Macro]);
                    $aSourceFiles = $this->getSourceParam($oGui, $oGtfFmwParser, $sDirSource, $bUsePattern);
                    // Test si le fichier source est un .zip.
                    $bZipSource = false;
                    if (strtolower(pathinfo($sSourceFilePath, PATHINFO_EXTENSION)) == 'zip') {
                        $bZipSource = true;
                        // Si le fichier n'est pas trouvé alors il se trouve dans le répertoire upload.
                        if (!file_exists($sSourceFilePath))
                            $sSourceFilePath = $this->aProperties["upload_dir"] . "/" . $sSourceFilePath;
                    }
                    // Formatage des fichiers sources.
                    foreach ($aSourceFiles as $iIndex => $sSourceFile) {
                        // Si le fichier source est un .zip -> paramètre publié = [chemin de l'archive zip]/[nom du fichier].
                        if ($bZipSource) {
                            if ($this->bFmeServer) {
                                $sSourceFile = str_replace($sDirSource, '', $sSourceFile);
                                $aSourceFiles[$iIndex] = pathinfo($sSourceFilePath, PATHINFO_BASENAME) . '/' . $sSourceFile;
                            }
                            else
                                $aSourceFiles[$iIndex] = '""' . str_replace($sDirSource, $sSourceFilePath, $sSourceFile) . '""';
                            // Supprime l'archive décompressée.
                            clearDir($sDirSource);
                        }
                        else if ($this->bFmeServer)
                            $aSourceFiles[$iIndex] = pathinfo($sSourceFile, PATHINFO_BASENAME);
                        else
                            $aSourceFiles[$iIndex] = '""' . $sSourceFile . '""';
                        // Supprime les doubles slashes.
                        $aSourceFiles[$iIndex] = str_replace('//', '/', $aSourceFiles[$iIndex]);
                    }
                    // Paramètre au format chaine de caractère.
                    if ($this->bFmeServer)
                        $this->aParametre[$oGui->sDefault_Macro] = implode('|', $aSourceFiles);
                    else
                        $this->aParametre[$oGui->sDefault_Macro] = '"' . implode('<space>', $aSourceFiles) . '"';
                }
            }
            if ($oGui->bIsDest) {
                $sDest = $this->aParametre[$oGui->sDefault_Macro];
                $this->updateDestParam($oGui, $oGtfFmwParser, $sDest);
            }

            if ($this->aParametre[$oGui->sDefault_Macro] != "") {

                if (!$oGui->bIsSource) {
                    // Protection des guillemets de chaque valeur pour qu'ils soient correctement envoyé à traitement.tcl
                    // Cette protection s'effectue uniquement pour les paramètre qui ne sont pas sources (un paramètre source est de ce type : """"param1"" ""param2"" ""param3"""" : les guillemets seraient donc protégé et cela fait planter l'exécution)
                    //$this->aParametre[$oGui->sDefault_Macro] = str_replace('"', '""', $this->aParametre[$oGui->sDefault_Macro]);
                }
                $sGTFOutput = $this->formatGuiOccurrence($oGui);
                $aFmeParameters[$oGui->sDefault_Macro] = $sGTFOutput;
            }
        }
        return $aFmeParameters;
    }
    
    /**
     * Cryptage d'une archive .zip.
     * @param {string} $sZipFilePath Path to the zip archive File.
     * @param {string} $sPassword Password for the zip archive.
     * @return boolean
     */
    function encryptZipArchiveFiles($sZipFilePath, $sPassword) {
        $zip = new ZipArchive;
        $res = $zip->open($sZipFilePath);
        if ($res === true) {
            for ($i = 0; $i < $zip->numFiles; $i++) {
                $zip->setEncryptionIndex($i, ZipArchive::EM_AES_256, $sPassword);
            }
            $zip->close();
            return true;
        }
        else
            return false;
    }
    
    /**
     * Return the path for the log file.
     * @param {string} $sResultDirectoryPath Path to the result directory.
     * @return string
     */
    function getFmeLogFilePath($sResultDirectoryPath) {
        return $sResultDirectoryPath . "/log/fme_" . $this->iDemandeId . ".log";
    }
    
    /**
     * Return the record for a job on Fme Server.
     * @param {string} $iJobId The id of the job.
     * @param {object} $oFmeServer FmeServer object.
     * @return object
     */
    function getFmeServerJobRecord($iJobId, $oFmeServer) {
        return $oFmeServer->serviceRequest('transformations/jobs/id/' . $iJobId, 'get');
    }
    
    /**
     * Download Result and log files of a job on Fme Server.
     * @param {number} $iJobId The od of the job.
     * @param {object} $oFmeServer FmeServer object.
     * @return object
     */
    function downloadFmeServerJobResult($iJobId, $oFmeServer, $aDestinationFiles, $sResource, $sPath) {
        $sZipFile = '';
        // Téléchargement du fichier de log.
        if ($oFmeServer->getJobLog($this->oJobResult->id, $this->getFmeLogFilePath($this->sResultDirectoryPath)) === false)
            return false;
        // Sauve l'url de log fme.
        $this->sLogFme = $this->getFmeLogFilePath($this->sResultDirectoryName);
        $sSql = "UPDATE " . $this->aProperties['schema_gtf'] . ".order SET log_url='" . $this->sLogFme . "' WHERE order_id=" . $this->iDemandeId;
        $this->oBd->execute($sSql);
        // Téléchargement des fichiers de résultat (si traitement).
        if ($this->oJobResult->status == 'SUCCESS') {
            // Télécharge l'archive .zip contenant tous les fichiers de résultat.
            if (!empty($aDestinationFiles)) {
                if ($oFmeServer->downloadResourceFile($sResource, $sPath . '/' . $this->sResultDirectoryName . '.zip', $this->sResultDirectoryPath . '/' . $this->iDemandeId . '.zip') === false)
                    return false;
                $sZipFile = $this->sResultDirectoryName . '/' . $this->iDemandeId . '.zip';
                // Paramètres de cryptage de l'utilisateur.
                $this->initUserEncryptionParameters();
                // Cryptage de l'archive .zip de résultat.
                if ($this->bDataEncrypt)
                    $this->encryptZipArchiveFiles($this->sResultDirectoryPath . '/' . $this->iDemandeId . '.zip', $this->sSecretKey);
            }
            // Log. du résultat
            if (!empty($this->oJobResult->result))
                $sStatusMessage = $this->oJobResult->result->statusMessage;
            else
                $sStatusMessage = $this->oJobResult->statusMessage;
            writeToLog(INFO_INFORM . 'FME| ' . $sStatusMessage, $this->sRobotLogFile);
            writeToLog(INFO_SUCCESSFUL_TREATMENT, $this->sRobotLogFile);
            /*
            // Télécharge tous les fichiers de résultat.
            foreach($aDestinationFiles as $sParamName) {
                $sFileName = $aFmeParameters[$sParamName];
                $oFmeServer->downloadResourceFile($sResource, $sPath . '/' . $sFileName, $this->sResultDirectoryPath . '/' . $sFileName);
            }
            // Tous les fichiers de résultat dans une archive .zip.
            $sZipFile = $this->zipResultFiles();
            */
            return $sZipFile;
        }
        else {
            if (!empty($this->oJobResult->result))
                $sStatusMessage = $this->oJobResult->result->statusMessage;
            else
                $sStatusMessage = $this->oJobResult->statusMessage;
            writeToLog(INFO_INFORM . 'FME| ' . $sStatusMessage, $this->sRobotLogFile);
            return false;
        }
    }
}
?>
